@@ -6,7 +6,8 @@ import claripy
 
 from ...storage.memory_mixins.paged_memory.pages.multi_values import MultiValues
 from ...knowledge_plugins.key_definitions import LiveDefinitions
-from ...knowledge_plugins.key_definitions.atoms import Atom, GuardUse, Register, MemoryLocation
+from ...knowledge_plugins.key_definitions.atoms import Atom, GuardUse, Register, MemoryLocation, FunctionCall, \
+    ConstantSrc
 from ...knowledge_plugins.key_definitions.definition import Definition
 from ...knowledge_plugins.key_definitions.environment import Environment
 from ...knowledge_plugins.key_definitions.tag import InitialValueTag, ParameterTag, Tag
@@ -50,10 +51,11 @@ class ReachingDefinitionsState:
     """
 
     __slots__ = ('arch', '_subject', '_track_tmps', 'analysis', 'current_codeloc', 'codeloc_uses', 'live_definitions',
-                 'all_definitions', '_canonical_size', 'heap_allocator', '_environment', )
+                 'all_definitions', '_canonical_size', 'heap_allocator', '_environment', '_track_calls',
+                 '_track_consts', )
 
-    def __init__(self, arch: archinfo.Arch, subject: Subject, track_tmps: bool=False,
-                 analysis: Optional['ReachingDefinitionsAnalysis']=None, rtoc_value=None,
+    def __init__(self, arch: archinfo.Arch, subject: Subject, track_tmps: bool=False, track_calls: bool=False,
+                 track_consts: bool=False, analysis: Optional['ReachingDefinitionsAnalysis']=None, rtoc_value=None,
                  live_definitions: Optional[LiveDefinitions]=None, canonical_size: int=8,
                  heap_allocator: HeapAllocator=None,
                  environment: Environment=None):
@@ -62,6 +64,8 @@ class ReachingDefinitionsState:
         self.arch = arch
         self._subject = subject
         self._track_tmps = track_tmps
+        self._track_calls = track_calls
+        self._track_consts = track_consts
         self.analysis = analysis
         self._canonical_size: int = canonical_size
 
@@ -289,6 +293,8 @@ class ReachingDefinitionsState:
             self.arch,
             self._subject,
             track_tmps=self._track_tmps,
+            track_calls=self._track_calls,
+            track_consts=self._track_consts,
             analysis=self.analysis,
             live_definitions=self.live_definitions.copy(),
             canonical_size=self._canonical_size,
@@ -416,5 +422,30 @@ class ReachingDefinitionsState:
         kinda_definition = Definition(atom, code_loc)
 
         if self.dep_graph is not None:
+            self.dep_graph.add_node(kinda_definition)
             for used in self.codeloc_uses:
                 self.dep_graph.add_edge(used, kinda_definition)
+
+    def mark_call(self, code_loc: CodeLocation, target):
+        self._cycle(code_loc)
+        atom = FunctionCall(target, code_loc)
+        kinda_definition = Definition(atom, code_loc)
+
+        if self.dep_graph is not None and self._track_calls:
+            self.dep_graph.add_node(kinda_definition)
+            for used in self.codeloc_uses:
+                self.dep_graph.add_edge(used, kinda_definition)
+            self.codeloc_uses.clear()
+            self.codeloc_uses.add(kinda_definition)
+            self.live_definitions.uses_by_codeloc[code_loc].clear()
+            self.live_definitions.uses_by_codeloc[code_loc].add(kinda_definition)
+
+    def mark_const(self, code_loc: CodeLocation, const):
+        self._cycle(code_loc)
+        atom = ConstantSrc(const)
+        kinda_definition = Definition(atom, code_loc)
+
+        if self.dep_graph is not None and self._track_consts:
+            self.dep_graph.add_node(kinda_definition)
+            self.codeloc_uses.add(kinda_definition)
+            self.live_definitions.uses_by_codeloc[code_loc].add(kinda_definition)
