@@ -448,7 +448,7 @@ class Tracer(ExplorationTechnique):
                 if self._compare_addr(self._trace[idx + 1], succ.addr):
                     res.append(succ)
                 else:
-                    *_, last_description = succ.history.descriptions
+                    last_description = succ.history.descriptions[-1]
                     if 'Unicorn' in last_description:
                         # A new state was created in SimEngineUnicorn. Check every recent basic block to see if any
                         # match the next expected index
