@@ -8,8 +8,9 @@ from .labels import Labels
 from .cfg import CFGManager
 from .xrefs import XRefManager
 from .plugin import KnowledgeBasePlugin
-from .sync import SynchronizationManager
+from .sync import SyncController
 from .patches import PatchManager
 from .key_definitions import KeyDefinitionManager
 from .propagations import Propagations
 from .structured_code import StructuredCodeManager
+from .types import TypesStore
