@@ -1,5 +1,5 @@
 # pylint:disable=abstract-method
-from typing import Iterable, Tuple, Any, Optional
+from typing import Iterable, Tuple, Dict, Any, Optional
 
 import claripy
 
@@ -127,6 +127,15 @@ class MemoryMixin(SimStatePlugin):
         """
         raise NotImplementedError()
 
+    def _merge_labels(self, labels: Iterable[Dict], **kwargs) -> Optional[Dict]:
+        """
+        Override this method to provide label merging support.
+
+        :param labels:          A collection of labels.
+        :return:                The merged label, or None to skip merging of the current label.
+        """
+        raise NotImplementedError()
+
     def replace_all(self, old: claripy.ast.BV, new: claripy.ast.BV):
         raise NotImplementedError()
 
@@ -156,6 +165,7 @@ from .convenient_mappings_mixin import ConvenientMappingsMixin
 from .default_filler_mixin import DefaultFillerMixin, SpecialFillerMixin, ExplicitFillerMixin
 from .dirty_addrs_mixin import DirtyAddrsMixin
 from .hex_dumper_mixin import HexDumperMixin
+from .label_merger_mixin import LabelMergerMixin
 from .multi_value_merger_mixin import MultiValueMergerMixin
 from .name_resolution_mixin import NameResolutionMixin
 from .simplification_mixin import SimplificationMixin
@@ -305,6 +315,7 @@ class LabeledMemory(
     ListPagesWithLabelsMixin,
     DefaultFillerMixin,
     TopMergerMixin,
+    LabelMergerMixin,
     PagedMemoryMixin,
 ):
     """
